package co.cr.apcnet.homealarm;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import android.app.Activity;
import android.hardware.Camera;
import android.media.CamcorderProfile;
import android.media.MediaRecorder;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.widget.Toast;


public class CaptureActivity extends Activity implements SurfaceHolder.Callback {
	private static boolean captureActivityIsOpen;
	public final String TAG = "HomeAlarm";
    
    
	private Camera mCamera;
    
    private boolean isPreview = false;
    private MediaRecorder mMediaRecorder;
    private SurfaceView mSurfaceView = null;
    private SurfaceHolder mHolder=null;
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.v(TAG, "onCreate()");
        setContentView(R.layout.capture);
        captureActivityIsOpen=true;
        mSurfaceView = (SurfaceView)findViewById(R.id.captureSurfaceView);
        mHolder = mSurfaceView.getHolder();
        mHolder.addCallback(this);
        new Handler().postDelayed(new Runnable() {
            @Override
            public void run() {
                //finish is here
            	finish();
            }
        }, 60000);
        
    }
    
	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		// TODO Auto-generated method stub
		Log.v(TAG, "surfaceCreated()");
		if(mCamera == null) {
            mCamera = Camera.open();
            isPreview = true;
        }
        try {
            mCamera.setPreviewDisplay(mHolder);
            mCamera.startPreview();
        } catch (IOException e) {
            Log.d(TAG, "Error setting camera preview: " + e.getMessage());
        }
        Prepare();
	}

	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width,
			int height) {
		Log.v(TAG, "surfaceChanged()");
		if (mHolder.getSurface() == null){
            // preview surface does not exist
            return;
        }

        // stop preview before making changes
        try {
            mCamera.stopPreview();
        } catch (Exception e){
            // ignore: tried to stop a non-existent preview
        }

        // set preview size and make any resize, rotate or
        // reformatting changes here

        // start preview with new settings
        try {
            mCamera.setPreviewDisplay(mHolder);
            mCamera.startPreview();

        } catch (Exception e){
            Log.d(TAG, "Error starting camera preview: " + e.getMessage());
        }
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		// TODO Auto-generated method stub
		Log.v(TAG, "surfaceDestroyed()");
	}
	

	@Override
    protected void onPause() {
        super.onPause();
        Log.v(TAG, "onPause()");
        releaseMediaRecorder();       // if you are using MediaRecorder, release it first
        releaseCamera();              // release the camera immediately on pause event
        captureActivityIsOpen=false;
        
    }
	
	
	@Override
	protected void onResume() {
		// TODO Auto-generated method stub
		super.onResume();
		captureActivityIsOpen=true;
	}

	@Override
	public void onBackPressed()
	{
	     // code here to show dialog
	     super.onBackPressed();  // optional depending on your needs
	     captureActivityIsOpen=false;
	}
	
	public void Prepare(){
		Log.v(TAG, "Prepare()");
		if (isPreview){

            releaseCamera();
            isPreview = false;

        }
		if (prepareVideoRecorder()) {

            mMediaRecorder.start();

        } else {

            releaseMediaRecorder();

        }
	}
	
	public static boolean captureActivityIsOpen() {
		return captureActivityIsOpen;

		}
    
	public static void captureActivityIsOpen(boolean captureActivityIsOpen) {
		//DayView.captureActivityIsOpen = captureActivityIsOpen;

		}
	
    private boolean prepareVideoRecorder(){

      if(Camera.getNumberOfCameras()<=0){
          Toast.makeText(this, "No Camera.", Toast.LENGTH_LONG).show();
          finish();
      }else{
          Toast.makeText(this, "Connect to Camera.", Toast.LENGTH_LONG).show();
      }

      mCamera = Camera.open();

      //mute
      mCamera.enableShutterSound(false);

      mMediaRecorder = new MediaRecorder();

      // Step 1: Unlock and set camera to MediaRecorder
      mCamera.unlock();
      mMediaRecorder.setCamera(mCamera);

      mMediaRecorder.setVideoSource(MediaRecorder.VideoSource.DEFAULT);
      mMediaRecorder.setAudioSource(MediaRecorder.AudioSource.DEFAULT);

      mMediaRecorder.setProfile(CamcorderProfile.get(CamcorderProfile.QUALITY_HIGH));
      mMediaRecorder.setOutputFile(getOutputMediaFile().toString());

      mMediaRecorder.setPreviewDisplay(mSurfaceView.getHolder().getSurface());

      try {
          mMediaRecorder.prepare();
      } catch (IllegalStateException e) {
          Log.d(TAG, "IllegalStateException preparing MediaRecorder: " + e.getMessage());
          releaseMediaRecorder();
          return false;
      } catch (IOException e) {
          Log.d(TAG, "IOException preparing MediaRecorder: " + e.getMessage());
          releaseMediaRecorder();
          return false;
      }

      return true;

  }
  private void releaseMediaRecorder(){
      if (mMediaRecorder != null) {
          mMediaRecorder.reset();
          mMediaRecorder.release();
          mMediaRecorder = null;
          mCamera.lock();
      }
  }

  private void releaseCamera(){
  	Toast.makeText(this, "Entrando Release",Toast.LENGTH_SHORT).show();
      
      if(mCamera!=null){
          mCamera.release();
          mCamera = null;
      }
  }
  /** Create a File for saving an image or video */
  private static File getOutputMediaFile(){

  	File path = new File (Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES) + "/misvideos/");

      // Create a media file name
      String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());

      File mediaFile;
      mediaFile = new File(path.getPath() + File.separator + "VID_" + timeStamp + ".mp4");

      return mediaFile;
  }
  class MediaPrepareTask extends AsyncTask<Void, Void, Boolean> {

      @Override
      protected Boolean doInBackground(Void... voids) {
          // initialize video camera
          if (prepareVideoRecorder()) {
              // Camera is available and unlocked, MediaRecorder is prepared,
              // now you can start recording
              mMediaRecorder.start();
          } else {
              // prepare didn't work, release the camera
              releaseMediaRecorder();
              return false;
          }
          return true;
      }

      @Override
      protected void onPostExecute(Boolean result) {
          if (!result) {
          	CaptureActivity.this.finish();
          }
          // inform the user that recording has started

      }
  }
}