package co.cr.apcnet.homealarm;


import android.app.Activity;
import android.app.ActivityManager;
import android.app.ActivityManager.RunningServiceInfo;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.hardware.Camera;
import android.hardware.Camera.Parameters;
import android.hardware.Camera.Size;
import android.os.Bundle;
import android.os.Handler;
import android.view.Menu;
import android.view.MenuItem;

import android.os.AsyncTask;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;
import android.widget.ToggleButton;


import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;


import java.io.ByteArrayOutputStream;
import java.util.Comparator;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.TreeSet;

public class MainActivity extends Activity {
	public static final String TAG = "HomeAlarm";
	private String web_service = "http://192.168.0.191/arduino/digital/6";
	private boolean mov_on = false;
	private ToggleButton tb_on_off;
	
	private static Context mContext;
	protected Sound mSound;
	private final float MIN_FREQUENCY_MHZ = 2401; 
	private final float MAX_FREQUENCY_MHZ = 2463;
	private final float MIN_AMPLITUDE_DBM = -100;
	private final float MAX_AMPLITUDE_DBM = -40;
	
	private boolean mIsServiceRunning = false;
    private Button mBackgroundButton;
    SharedPreferences mSharedPreferences;
    
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		tb_on_off = (ToggleButton) findViewById(R.id.toggleButton_on_off);
		mContext = this;
		
		mBackgroundButton = (Button) findViewById(R.id.backgroundButton);
		
		Sound.initialize();
		mSound = new Sound(MAX_FREQUENCY_MHZ, MIN_FREQUENCY_MHZ, MAX_FREQUENCY_MHZ, 0, MIN_AMPLITUDE_DBM, MAX_AMPLITUDE_DBM, 0, R.raw.acheta);
		final Handler handler = new Handler();
		Timer timer = new Timer();
		TimerTask task = new TimerTask() {       
		     @Override
		     public void run() {
		       handler.post(new Runnable() {
		          public void run() {  
		             new PerformBackgroundTask().execute();
		          }
		        });
		      }
		};
		timer.schedule(task, 0, 1000);
		
		if (!initialize()) {
            Toast.makeText(this, "Can not initialize parameters", Toast.LENGTH_LONG).show();
        }
		
	}
	
	@Override
    public void onResume() {
        super.onResume();
        
        mIsServiceRunning = isServiceRunning();
        updateButton(mIsServiceRunning);
    }
    
    @Override
    public void onStop() {
        super.onStop();
        
        if (mIsServiceRunning) {
            finish();
        }
    }

	private void actualizaEstadoBoton() throws InterruptedException {
		if (mov_on) {
			tb_on_off.setChecked(true);
			mIsServiceRunning = isServiceRunning();
	        
	        if (mIsServiceRunning) {
                //doUnbindService();
                stopService(new Intent(this, BackgroundService.class));
                mIsServiceRunning = false;
            }
	        updateButton(mIsServiceRunning);
			mSound.startPlaying();
			if (CaptureActivity.captureActivityIsOpen() == false) {
				startActivity(new Intent(this , CaptureActivity.class));
				
			}
		} else {
			mIsServiceRunning = isServiceRunning();
	        if (!mIsServiceRunning) {
	        	if (CaptureActivity.captureActivityIsOpen() == false) {
	        		startService(new Intent(this, BackgroundService.class));
		            mIsServiceRunning = true;
				}
	        }
	        updateButton(mIsServiceRunning);
			tb_on_off.setChecked(false);
			mSound.stopPlaying();
		}
	}

	public void click(View v) {
		
	}
	
	public void onButtonClick(View view) {
		switch (view.getId()) {
        case R.id.settingsButton:
            startActivity(new Intent(this , SettingsActivity.class));
            break;
        case R.id.backgroundButton:
            if (!mIsServiceRunning) {
                //doBindService();
                startService(new Intent(this, BackgroundService.class));
                mIsServiceRunning = true;
            } else {
                //doUnbindService();
                stopService(new Intent(this, BackgroundService.class));
                mIsServiceRunning = false;
            }
            updateButton(mIsServiceRunning);
            break;
        }
	}
	
	public static Context getContext(){
	      return mContext;
	  }
	
	private boolean isServiceRunning() {
        ActivityManager manager = (ActivityManager) getSystemService(Context.ACTIVITY_SERVICE);
        for (RunningServiceInfo service : manager.getRunningServices(Integer.MAX_VALUE)) {
            if ((BackgroundService.class.getName()).equals(service.service.getClassName())) {
                return true;
            }
        }
        return false;
    }
	
	
	private void updateButton(boolean state) {
        if (state) {
            mBackgroundButton.setText(R.string.stop_running);
        } else {
            mBackgroundButton.setText(R.string.run_background);
        }
    }
	
	private boolean initialize() {
        mSharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        
       boolean firstRun = ! mSharedPreferences.contains("settings_camera");
       if (firstRun) {
           Log.v(TAG, "First run");
           
           SharedPreferences.Editor editor = mSharedPreferences.edit();
           
           int cameraNumber = Camera.getNumberOfCameras();
           Log.v(TAG, "Camera number: " + cameraNumber);
           
           /*
            * Get camera name set 
            */
           TreeSet<String> cameraNameSet = new TreeSet<String>();
           if (cameraNumber == 1) {
               cameraNameSet.add("back");
           } else if (cameraNumber == 2) {
               cameraNameSet.add("back");
               cameraNameSet.add("front");
           } else if (cameraNumber > 2) {           // rarely happen
               for (int id = 0; id < cameraNumber; id++) {
                   cameraNameSet.add(String.valueOf(id));
               }
           } else {                                 // no camera available
               Log.v(TAG, "No camrea available");
               Toast.makeText(this, "No camera available", Toast.LENGTH_SHORT).show();
               
               return false;
           }

           /* 
            * Get camera id set
            */
           String[] cameraIds = new String[cameraNumber];
           TreeSet<String> cameraIdSet = new TreeSet<String>();
           for (int id = 0; id < cameraNumber; id++) {
               cameraIdSet.add(String.valueOf(id));
           }
           
           /*
            * Save camera name set and id set
            */
           editor.putStringSet("camera_name_set", cameraNameSet);
           editor.putStringSet("camera_id_set", cameraIdSet);
           
           /*
            * Get and save camera parameters
            */
           for (int id = 0; id < cameraNumber; id++) {
               Camera camera = Camera.open(id);
               if (camera == null) {
                   String msg = "Camera " + id + " is not available";
                   Log.v(TAG, msg);
                   Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
                   
                   return false;
               }
               
               Parameters parameters = camera.getParameters();
               
               /*
                * Get and save preview sizes
                */
               List<Size> sizes = parameters.getSupportedPreviewSizes();
               
               TreeSet<String> sizeSet = new TreeSet<String>(new Comparator<String>() {
                   @Override
                   public int compare(String s1, String s2) {
                       int spaceIndex1 = s1.indexOf(" ");
                       int spaceIndex2 = s2.indexOf(" ");
                       int width1 = Integer.parseInt(s1.substring(0, spaceIndex1));
                       int width2 = Integer.parseInt(s2.substring(0, spaceIndex2));
                       
                       return width2 - width1;
                   }
               });
               for (Size size : sizes) {
                   sizeSet.add(size.width + " x " + size.height);
               }
               editor.putStringSet("preview_sizes_" + id, sizeSet);
               
               Log.v(TAG, sizeSet.toString());
               
               /*
                * Set default preview size, use camera 0
                */
               if (id == 0) {
                   Log.v(TAG, "Set default preview size");
                   
                   Size defaultSize = parameters.getPreviewSize();
                   editor.putString("settings_size", defaultSize.width + " x " + defaultSize.height);
               }
               
               /*
                * Get and save 
                */
               List<int[]> ranges = parameters.getSupportedPreviewFpsRange();
               TreeSet<String> rangeSet = new TreeSet<String>();
               for (int[] range : ranges) {
                   rangeSet.add(range[0] + " ~ " + range[1]);
               }
               editor.putStringSet("preview_ranges_" + id, rangeSet);
               
               if (id == 0) {
                   Log.v(TAG, "Set default fps range");
                   
                   int[] defaultRange = new int[2];
                   parameters.getPreviewFpsRange(defaultRange);
                   editor.putString("settings_range", defaultRange[0] + " ~ " + defaultRange[1]);
               }
               
               camera.release();
               
           }
           
           editor.putString("settings_camera", "0");
           editor.commit();
       }
       
       return true;
    }
	public class PerformBackgroundTask extends AsyncTask<Void, Void, Void>{

		@Override
		protected Void doInBackground(Void... params) {
			HttpClient httpclient = new DefaultHttpClient();
			try {
				HttpResponse response = httpclient.execute(new HttpGet(web_service));
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				response.getEntity().writeTo(out);
				out.close();

				String respuesta = out.toString();
				//Log.w(TAG,"Valor " + respuesta);
				//Si todo ha ido bien, recibiremos una respuesta del tipo "Pin DY set to X\r\n"
				//donde X ser 0 o 1 y DY es el Pin al que acceso.

				if (respuesta.endsWith("0\r\n")) {
					mov_on = false;
				} else {
					mov_on = true;
				}

			} catch (Exception e) {
				e.printStackTrace();
			}

			return null;
		}
		
		@Override
		protected void onPostExecute(Void aVoid) {
			mSound.stopPlaying();
			try {
				actualizaEstadoBoton();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

}
